@@ -35,11 +35,12 @@ import (
 )
 
 var (
-	flagYaml   bool
-	flagRaw    bool
-	flagSlurp  bool
-	flagComp   bool
-	flagStrict bool
+	flagYaml     bool
+	flagRaw      bool
+	flagSlurp    bool
+	flagComp     bool
+	flagStrict   bool
+	flagNoInline bool
 )
 
 func main() {
@@ -97,6 +98,8 @@ func main() {
 			flagSlurp = true
 		case "--strict":
 			flagStrict = true
+		case "--no-inline":
+			flagNoInline = true
 		default:
 			args = append(args, arg)
 		}
@@ -168,9 +171,13 @@ func main() {
 	}
 
 	if len(args) > 0 || flagSlurp {
-		writeOut := func(s string) { fmt.Println(s) }
-		writeErr := func(s string) { fmt.Fprintln(os.Stderr, s) }
-		exitCode := engine.Start(parser, args, flagSlurp, writeOut, writeErr)
+		opts := engine.Options{
+			Slurp:      flagSlurp,
+			WithInline: !flagNoInline,
+			WriteOut:   func(s string) { fmt.Println(s) },
+			WriteErr:   func(s string) { fmt.Fprintln(os.Stderr, s) },
+		}
+		exitCode := engine.Start(parser, args, opts)
 		if exitCode != 0 {
 			os.Exit(exitCode)
 		}
@@ -228,6 +235,7 @@ func main() {
 		commandInput:    commandInput,
 		searchInput:     searchInput,
 		search:          newSearch(),
+		searchCache:     newSearchCache(50), // Cache up to 50 search queries
 		spinner:         spinnerModel,
 	}
 
@@ -267,6 +275,7 @@ func main() {
 				firstOk = true
 				p.Send(nodeMsg{node: node})
 			}
+			parser.SkipWhitespace()
 		}
 	}()
 
@@ -303,6 +312,7 @@ type model struct {
 	commandInput          textinput.Model
 	searchInput           textinput.Model
 	search                *search
+	searchCache           *searchCache
 	yank                  bool
 	showHelp              bool
 	help                  viewport.Model
@@ -339,6 +349,7 @@ func (m *model) Init() tea.Cmd {
 
 func (m *model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 	if msg, ok := msg.(tea.WindowSizeMsg); ok {
+		oldTermWidth := m.termWidth
 		m.termWidth = msg.Width
 		m.termHeight = msg.Height
 		m.help.Width = m.termWidth
@@ -346,6 +357,10 @@ func (m *model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 		m.preview.Width = m.termWidth
 		m.preview.Height = m.termHeight - 1
 		Wrap(m.top, m.viewWidth())
+		// Only invalidate cache if terminal width changed and wrapping is enabled
+		if oldTermWidth != m.termWidth && m.wrap {
+			m.searchCache.invalidate()
+		}
 		m.redoSearch()
 	}
 
@@ -367,6 +382,9 @@ func (m *model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 		return m, tea.Quit
 
 	case nodeMsg:
+		// Invalidate search cache when new data arrives
+		m.searchCache.invalidate()
+
 		if m.wrap {
 			Wrap(msg.node, m.viewWidth())
 		}
@@ -380,7 +398,11 @@ func (m *model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 			m.top = msg.node
 			m.bottom = msg.node
 		} else {
-			scrollToBottom := m.cursorPointsTo() == m.bottom.Bottom()
+			to, ok := m.cursorPointsTo()
+			if !ok {
+				return m, nil
+			}
+			scrollToBottom := to == m.bottom.Bottom()
 			msg.node.Index = -1 // To fix the statusbar path (to show .key instead of [0].key).
 			m.bottom.Adjacent(msg.node)
 			m.bottom = msg.node
@@ -410,8 +432,8 @@ func (m *model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 			m.showCursor = true
 			if msg.Y < m.viewHeight() {
 				if m.cursor == msg.Y {
-					to := m.cursorPointsTo()
-					if to != nil {
+					to, ok := m.cursorPointsTo()
+					if ok {
 						if to.IsCollapsed() {
 							to.Expand()
 						} else {
@@ -659,6 +681,11 @@ func (m *model) handleYankKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 		_ = clipboard.WriteAll(m.cursorKey())
 	case key.Matches(msg, yankValueY, yankValueV):
 		_ = clipboard.WriteAll(m.cursorValue())
+	case key.Matches(msg, yankKeyValue):
+		k := m.cursorKey()
+		v := m.cursorValue()
+		keyValue := k + ": " + v
+		_ = clipboard.WriteAll(keyValue)
 	}
 	m.yank = false
 	return m, nil
@@ -731,7 +758,10 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 		m.recordHistory()
 
 	case key.Matches(msg, keyMap.NextSibling):
-		pointsTo := m.cursorPointsTo()
+		pointsTo, ok := m.cursorPointsTo()
+		if !ok {
+			return m, nil
+		}
 		var nextSibling *Node
 		if pointsTo.End != nil && pointsTo.End.Next != nil {
 			nextSibling = pointsTo.End.Next
@@ -746,7 +776,10 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 		m.recordHistory()
 
 	case key.Matches(msg, keyMap.PrevSibling):
-		pointsTo := m.cursorPointsTo()
+		pointsTo, ok := m.cursorPointsTo()
+		if !ok {
+			return m, nil
+		}
 		var prevSibling *Node
 		parent := pointsTo.Parent
 		if parent != nil && parent.End == pointsTo {
@@ -766,8 +799,8 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 		m.recordHistory()
 
 	case key.Matches(msg, keyMap.Collapse):
-		n := m.cursorPointsTo()
-		if n == nil {
+		n, ok := m.cursorPointsTo()
+		if !ok {
 			return m, nil
 		}
 		if n.HasChildren() && !n.IsCollapsed() {
@@ -781,26 +814,36 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 		m.recordHistory()
 
 	case key.Matches(msg, keyMap.Expand):
-		m.cursorPointsTo().Expand()
+		n, ok := m.cursorPointsTo()
+		if !ok {
+			return m, nil
+		}
+		n.Expand()
 		m.showCursor = true
 
 	case key.Matches(msg, keyMap.CollapseRecursively):
-		n := m.cursorPointsTo()
+		n, ok := m.cursorPointsTo()
+		if !ok {
+			return m, nil
+		}
 		if n.HasChildren() {
 			n.CollapseRecursively()
 		}
 		m.showCursor = true
 
 	case key.Matches(msg, keyMap.ExpandRecursively):
-		n := m.cursorPointsTo()
+		n, ok := m.cursorPointsTo()
+		if !ok {
+			return m, nil
+		}
 		if n.HasChildren() {
 			n.ExpandRecursively(0, math.MaxInt)
 		}
 		m.showCursor = true
 
 	case key.Matches(msg, keyMap.CollapseAll):
-		at := m.cursorPointsTo()
-		if at != nil {
+		at, ok := m.cursorPointsTo()
+		if ok {
 			m.collapsed = true
 			n := m.top
 			for n != nil {
@@ -818,7 +861,10 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 		}
 
 	case key.Matches(msg, keyMap.ExpandAll):
-		at := m.cursorPointsTo()
+		at, ok := m.cursorPointsTo()
+		if !ok {
+			return m, nil
+		}
 		m.collapsed = false
 		n := m.top
 		for n != nil {
@@ -832,8 +878,8 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 		m.selectNode(at)
 
 	case key.Matches(msg, keyMap.CollapseLevel):
-		at := m.cursorPointsTo()
-		if at != nil && at.HasChildren() {
+		at, ok := m.cursorPointsTo()
+		if ok && at.HasChildren() {
 			toLevel, _ := strconv.Atoi(msg.String())
 			at.CollapseRecursively()
 			at.ExpandRecursively(0, toLevel)
@@ -841,13 +887,17 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 		}
 
 	case key.Matches(msg, keyMap.ToggleWrap):
-		at := m.cursorPointsTo()
+		at, ok := m.cursorPointsTo()
+		if !ok {
+			return m, nil
+		}
 		m.wrap = !m.wrap
 		if m.wrap {
 			Wrap(m.top, m.viewWidth())
 		} else {
 			DropWrapAll(m.top)
 		}
+		m.searchCache.invalidate()
 		if at.Chunk != "" && at.Value == "" {
 			at = at.Parent
 		}
@@ -883,7 +933,10 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 		return m, m.open()
 
 	case key.Matches(msg, keyMap.Dig):
-		at := m.cursorPointsTo()
+		at, ok := m.cursorPointsTo()
+		if !ok {
+			return m, nil
+		}
 		if at.Kind == Err {
 			nextJson := at.FindNextNonErr()
 			if nextJson != nil {
@@ -902,7 +955,10 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 		m.createKeysIndex()
 
 	case key.Matches(msg, keyMap.GotoRef):
-		at := m.cursorPointsTo()
+		at, ok := m.cursorPointsTo()
+		if !ok {
+			return m, nil
+		}
 		value, isRef := isRefNode(at)
 		if isRef {
 			refPath, ok := jsonpath.ParseSchemaRef(value)
@@ -932,7 +988,10 @@ func (m *model) handleKey(msg tea.KeyMsg) (tea.Model, tea.Cmd) {
 
 	case key.Matches(msg, keyMap.GoBack):
 		if m.locationIndex > 0 {
-			at := m.cursorPointsTo()
+			at, ok := m.cursorPointsTo()
+			if !ok {
+				return m, nil
+			}
 			m.locationIndex--
 
 			loc := m.locationHistory[m.locationIndex]
@@ -976,8 +1035,8 @@ func (m *model) down() {
 	}
 	m.showCursor = true
 	m.cursor++
-	n := m.cursorPointsTo()
-	if n == nil {
+	_, ok := m.cursorPointsTo()
+	if !ok {
 		m.cursor--
 		return
 	}
@@ -990,8 +1049,8 @@ func (m *model) down() {
 }
 
 func (m *model) recordHistory() {
-	at := m.cursorPointsTo()
-	if at == nil {
+	at, ok := m.cursorPointsTo()
+	if !ok {
 		return
 	}
 	if at.Chunk != "" && at.Value == "" {
@@ -1058,6 +1117,9 @@ func (m *model) scrollBackward(lines int) {
 }
 
 func (m *model) scrollForward(lines int) {
+	if m.head == nil {
+		return
+	}
 	it := m.head
 	for it.Next != nil {
 		it = it.Next
@@ -1093,7 +1155,7 @@ func (m *model) prettyKey(node *Node, selected bool) []byte {
 	}
 }
 
-func (m *model) prettyPrint(node *Node, selected bool) string {
+func (m *model) prettyPrint(node *Node, isSelected, isRef bool) string {
 	var s string
 	if node.Chunk != "" {
 		s = node.Chunk
@@ -1102,14 +1164,17 @@ func (m *model) prettyPrint(node *Node, selected bool) string {
 	}
 
 	if len(s) == 0 {
-		if selected {
+		if isSelected {
 			return theme.CurrentTheme.Cursor(" ")
 		} else {
 			return s
 		}
 	}
 
-	style := theme.Value(node.Kind, selected)
+	style := theme.Value(node.Kind, isSelected)
+	if isRef {
+		style = theme.CurrentTheme.Ref
+	}
 
 	if indexes, ok := m.search.values[node]; ok {
 		var out strings.Builder
@@ -1156,8 +1221,9 @@ func (m *model) viewHeight() int {
 	return m.termHeight - 1
 }
 
-func (m *model) cursorPointsTo() *Node {
-	return m.at(m.cursor)
+func (m *model) cursorPointsTo() (*Node, bool) {
+	n := m.at(m.cursor)
+	return n, n != nil
 }
 
 func (m *model) at(pos int) *Node {
@@ -1221,8 +1287,11 @@ func (m *model) selectNode(n *Node) {
 }
 
 func (m *model) cursorPath() string {
+	at, ok := m.cursorPointsTo()
+	if !ok {
+		return ""
+	}
 	path := ""
-	at := m.cursorPointsTo()
 	for at != nil {
 		if at.Prev != nil {
 			if at.Chunk != "" && at.Value == "" {
@@ -1246,8 +1315,8 @@ func (m *model) cursorPath() string {
 }
 
 func (m *model) cursorValue() string {
-	at := m.cursorPointsTo()
-	if at == nil {
+	at, ok := m.cursorPointsTo()
+	if !ok {
 		return ""
 	}
 	parent := at.Parent
@@ -1306,8 +1375,8 @@ func (m *model) cursorValue() string {
 }
 
 func (m *model) cursorKey() string {
-	at := m.cursorPointsTo()
-	if at == nil {
+	at, ok := m.cursorPointsTo()
+	if !ok {
 		return ""
 	}
 	if at.IsWrap() {
@@ -1319,7 +1388,6 @@ func (m *model) cursorKey() string {
 		return v
 	}
 	return strconv.Itoa(at.Index)
-
 }
 
 func (m *model) findByPath(path []any) *Node {
@@ -1328,8 +1396,8 @@ func (m *model) findByPath(path []any) *Node {
 }
 
 func (m *model) currentTopNode() *Node {
-	at := m.cursorPointsTo()
-	if at == nil {
+	at, ok := m.cursorPointsTo()
+	if !ok {
 		return nil
 	}
 	for at.Parent != nil {
@@ -1339,12 +1407,18 @@ func (m *model) currentTopNode() *Node {
 }
 
 func (m *model) doSearch(s string) {
-	m.search = newSearch()
-
 	if s == "" {
 		return
 	}
 
+	// Check cache first
+	if cachedSearch, _, found := m.searchCache.get(s); found {
+		m.search = cachedSearch
+		return
+	}
+
+	m.search = newSearch()
+
 	code, ci := regexCase(s)
 	if ci {
 		code = "(?i)" + code
@@ -1408,6 +1482,8 @@ func (m *model) doSearch(s string) {
 		}
 	}
 
+	m.searchCache.put(s, re, m.search)
+
 	m.selectSearchResult(0)
 }
 
@@ -1436,8 +1512,8 @@ func (m *model) redoSearch() {
 }
 
 func (m *model) createKeysIndex() {
-	at := m.cursorPointsTo()
-	if at == nil {
+	at, ok := m.cursorPointsTo()
+	if !ok {
 		return
 	}
 	root := at.Root()
@@ -1500,8 +1576,8 @@ func (m *model) open() tea.Cmd {
 		engine.FilePath,
 	)
 	if command[0] == "vi" || command[0] == "vim" {
-		at := m.cursorPointsTo()
-		if at != nil {
+		at, ok := m.cursorPointsTo()
+		if ok {
 			tail := command[1:]
 			command = append([]string{command[0]}, fmt.Sprintf("+%d", at.LineNumber))
 			command = append(command, tail...)
